@@ -22,12 +22,13 @@
 import argparse
 import datetime
 import os
+import shlex
 import subprocess
 
-try:
-    import configparser
-except ImportError:  # Python 3
-    import ConfigParser as configparser  # NOQA
+try:  # For Python 3
+    from configparser import NoOptionError
+except ImportError:
+    from ConfigParser import NoOptionError  # NOQA
 
 import aaargh
 import prettytable
@@ -37,8 +38,10 @@ from .i18n import (_, N_)
 from . import _version
 from . import utils
 
-APP = aaargh.App(description=__doc__.splitlines()[0].split('-', 1)[1],
-                 epilog=_('Please report bugs to jnrowe@gmail.com'))
+APP = aaargh.App(
+    description=_('Simple time tracking for simple people'),
+    epilog=_('Please report bugs to https://github.com/JNRowe/issues')
+)
 
 
 # pylint: disable-msg=R0903
@@ -131,11 +134,12 @@ def start_time_typecheck(string):
 
 
 @APP.cmd(help=_('check storage consistency'))
-def fsck(directory, backup):
+def fsck(directory, backup, config):
     """Check storage consistency.
 
     :param str directory: Directory to read events from
     :param bool backup: Whether to create backup files
+    :param ConfigParser config: Configuration data
 
     """
     with Events.context(directory, backup) as events:
@@ -143,7 +147,7 @@ def fsck(directory, backup):
         for event in events[1:]:
             if not last.start + last.delta <= event.start:
                 print(utils.fail(_('Overlap:')))
-                print('  %r' % last)
+                print(utils.warn('  %r' % last))
                 print('  %r' % event)
             last = event
 
@@ -152,11 +156,13 @@ def fsck(directory, backup):
 @APP.cmd_arg('-n', '--new', action='store_true', help=_('start a new task'))
 @APP.cmd_arg('-t', '--time', metavar='time', default='',
              help=_('set start time'), type=start_time_typecheck)
-def start(directory, backup, task, new, time):
+@utils.write_current
+def start(directory, backup, config, task, new, time):
     """Start task.
 
     :param str directory: Directory to read events from
     :param bool backup: Whether to create backup files
+    :param ConfigParser config: Configuration data
     :param str task: Task name to operate on
     :param bool new: Create a new task
     :param datetime.datetime time: Task start time
@@ -164,7 +170,6 @@ def start(directory, backup, task, new, time):
     """
     with Events.context(directory, backup) as events:
         events.start(task, new, time)
-    open('%s/.current' % directory, 'w').write(task)
 
 
 @APP.cmd(help=_('stop task'))
@@ -173,11 +178,13 @@ def start(directory, backup, task, new, time):
              help=_('read closing message from file'))
 @APP.cmd_arg('--amend', action='store_true',
              help=_('amend previous stop entry'))
-def stop(directory, backup, message, file, amend):
+@utils.remove_current
+def stop(directory, backup, config, message, file, amend):
     """Stop task.
 
     :param str directory: Directory to read events from
     :param bool backup: Whether to create backup files
+    :param ConfigParser config: Configuration data
     :param str message: Message to assign to event
     :param str file: Filename to read message from
     :param bool amend: Amend a previously stopped event
@@ -197,8 +204,6 @@ def stop(directory, backup, message, file, amend):
     event = events.last()
     print(_('Task %s running for %s') % (event.task,
                                          str(event.delta).split('.')[0]))
-    if os.path.isfile('%s/.current' % directory):
-        os.unlink('%s/.current' % directory)
 
 
 @APP.cmd(help=_('switch to another task'), parents=[task_parser])
@@ -207,11 +212,13 @@ def stop(directory, backup, message, file, amend):
              help=_('closing message for current task'))
 @APP.cmd_arg('-F', '--file', metavar='file', type=argparse.FileType(),
              help=_('read closing message for current task from file'))
-def switch(directory, backup, task, new, message, file):
+@utils.write_current
+def switch(directory, backup, config, task, new, message, file):
     """Complete last task and start new one.
 
     :param str directory: Directory to read events from
     :param bool backup: Whether to create backup files
+    :param ConfigParser config: Configuration data
     :param str task: Task name to operate on
     :param bool new: Create a new task
     :param str message: Message to assign to event
@@ -229,7 +236,6 @@ def switch(directory, backup, task, new, message, file):
         events.start(task, new)
     print(_('Task %s running for %s') % (event.task,
                                          str(event.delta).split('.')[0]))
-    open('%s/.current' % directory, 'w').write(task)
 
 
 @APP.cmd(help=_('run command with timer'), parents=[task_parser])
@@ -240,11 +246,12 @@ def switch(directory, backup, task, new, message, file):
 @APP.cmd_arg('-F', '--file', metavar='file', type=argparse.FileType(),
              help=_('read closing message from file'))
 @APP.cmd_arg('-c', '--command', metavar='command', help=_('command to run'))
-def run(directory, backup, task, new, time, message, file, command):
+def run(directory, backup, config, task, new, time, message, file, command):
     """Run timed command.
 
     :param str directory: Directory to read events from
     :param bool backup: Whether to create backup files
+    :param ConfigParser config: Configuration data
     :param str task: Task name to operate on
     :param bool new: Create a new task
     :param datetime.datetime time: Task start time
@@ -277,6 +284,36 @@ def run(directory, backup, task, new, time, message, file, command):
     os.unlink('%s/.current' % directory)
 
 
+@APP.cmd(help=_('run predefined command with timer'))
+@APP.cmd_arg('-t', '--time', metavar='time', default='',
+             help=_('set start time'), type=start_time_typecheck)
+@APP.cmd_arg('-m', '--message', metavar='message', help=_('closing message'))
+@APP.cmd_arg('-F', '--file', metavar='file', type=argparse.FileType(),
+             help=_('read closing message from file'))
+@APP.cmd_arg('wrapper', default='default', help=_('wrapper name'))
+def wrapper(directory, backup, config, time, message, file, wrapper):
+    """Run predefined timed command.
+
+    :param str directory: Directory to read events from
+    :param bool backup: Whether to create backup files
+    :param ConfigParser config: Configuration data
+    :param datetime.datetime time: Task start time
+    :param str message: Message to assign to event
+    :param str file: Filename to read message from
+    :param str wrapper: Run wrapper to execute
+
+    """
+    try:
+        command = config.get('run wrappers', wrapper)
+    except NoOptionError:
+        raise ValueError(_('No such wrapper %r') % wrapper)
+    parser = argparse.ArgumentParser(parents=[task_parser, ])
+    parser.add_argument('-c', '--command')
+    args = parser.parse_args(shlex.split(command))
+    run(directory, backup, config, args.task, False, time, message, file,
+        args.command)
+
+
 # pylint: disable-msg=C0103
 output_parser = argparse.ArgumentParser(add_help=False)
 output_group = output_parser.add_mutually_exclusive_group()
@@ -293,11 +330,13 @@ output_group.add_argument('--human', action='store_true',
              help=_('field to sort by'))
 @APP.cmd_arg('-r', '--reverse', action='store_true',
              help=_('reverse sort order'))
-def report(directory, backup, task, duration, sort, reverse, html, human):
+def report(directory, backup, config, task, duration, sort, reverse, html,
+           human):
     """Report time tracking data.
 
     :param str directory: Directory to read events from
     :param bool backup: Whether to create backup files
+    :param ConfigParser config: Configuration data
     :param str task: Task name to operate on
     :param str duration: Time window to filter on
     :param str sort: Key to sort events on
@@ -373,11 +412,12 @@ def last(directory, backup):
          parents=[duration_parser, task_parser])
 @APP.cmd_arg('-r', '--rate', metavar='rate',
              help=_('hourly rate for task output'))
-def ledger(directory, backup, task, duration, rate):
+def ledger(directory, backup, config, task, duration, rate):
     """Generate ledger compatible data file.
 
     :param str directory: Directory to read events from
     :param bool backup: Whether to create backup files
+    :param ConfigParser config: Configuration data
     :param str task: Task name to operate on
     :param str duration: Time window to filter on
     :param str rate: Rate to assign hours in report
@@ -406,15 +446,10 @@ def ledger(directory, backup, task, duration, rate):
 
 def main():
     """Main script."""
-    configs = [os.path.dirname(__file__) + '/config', ]
-    for s in os.getenv('XDG_CONFIG_DIRS', '/etc/xdg').split(':'):
-        p = s + '/rdial/config'
-        if os.path.isfile(p):
-            configs.append(p)
-    configs.append(utils.xdg_config_location() + '/config')
-    configs.append(os.path.abspath('.rdialrc'))
-    cfg = configparser.SafeConfigParser()
-    cfg.read(configs)
+    parser = argparse.ArgumentParser(add_help=False)
+    parser.add_argument('--config')
+    args, remaining = parser.parse_known_args()
+    cfg = utils.read_config(parser, args.config)
 
     if not cfg.getboolean('rdial', 'colour') or os.getenv('NO_COLOUR'):
         utils._colourise = lambda s, colour: s
@@ -435,12 +470,15 @@ def main():
             help=_('directory to read/write to'))
     APP.arg('--no-backup', dest='backup', action='store_true',
             help=_('do not write data file backups'))
+    APP.arg('--config', metavar='file',
+            help=_('file to read configuration data from'))
     APP.defaults(backup=not cfg.getboolean('rdial', 'backup'),
                  directory=cfg.get('rdial', 'directory',
                                    vars={'xdg_data_location':
-                                         utils.xdg_data_location()}))
+                                         utils.xdg_data_location()}),
+                 config=cfg)
     try:
-        APP.run()
+        APP.run(remaining)
     except utils.RdialError as error:
         print(utils.fail(error.message))
         return 2
